@@ -6,6 +6,7 @@ import inspect
 import logging
 import typing
 import types
+import warnings
 from inspect import isclass, signature
 from inspect import Parameter
 from typing import cast
@@ -257,9 +258,13 @@ class Endpoint(IEndpoint, metaclass=EndpointMetaclass):
         **kwargs: typing.Any
     ) -> None:
         cors_policy = cors_policy or cls.cors_policy or cors.DefaultPolicy
-        method = method or cls.method
-        if method is None:
-            raise ValueError("The `method` parameter can not be None.")
+        methods = getattr(cls, 'methods', None) or [method or cls.method]
+        if cls.method:
+            warnings.warn(
+                "The Endpoint.method attribute is deprecated",
+                DeprecationWarning
+            )
+            methods = [cls.method]
         if not str.startswith(base_path, '/'):
             raise ValueError("The `base_path` parameter must start with a slash.")
         base_path = str.rstrip(base_path, '/')
@@ -270,11 +275,11 @@ class Endpoint(IEndpoint, metaclass=EndpointMetaclass):
         cls = cls.new(
             __module__=cls.__module__,
             cors_policy=cors_policy,
-            method=method
+            methods=methods
         )
 
         endpoint = cls.as_handler(
-            method = method,
+            methods = methods,
             path = base_path,
             request_handler = request_handler
         )
@@ -297,7 +302,7 @@ class Endpoint(IEndpoint, metaclass=EndpointMetaclass):
         app.add_api_route(
             dependencies=[fastapi.Depends(cors_policy)],
             path=base_path,
-            methods=[method],
+            methods=methods,
             endpoint=endpoint,
             responses={
                 **cls.get_responses(),
@@ -329,7 +334,7 @@ class Endpoint(IEndpoint, metaclass=EndpointMetaclass):
     @classmethod
     def as_handler(
         cls,
-        method: str,
+        methods: list[str],
         path: str,
         request_handler: typing.Optional[
             typing.Callable[..., IEndpoint.ResponseTypes]
@@ -344,9 +349,7 @@ class Endpoint(IEndpoint, metaclass=EndpointMetaclass):
         cors_policy = cls.cors_policy
         origin_header = cors_policy.origin_header
         negotiation = cls.negotiation
-        if method in {"POST", "PUT", "PATCH"}:
-            negotiation = cls.negotiation.with_body
-        if method == "OPTIONS":
+        if methods == ["OPTIONS"]:
             cors_policy = cors_policy.as_options
 
         async def handle(
@@ -420,9 +423,8 @@ class Endpoint(IEndpoint, metaclass=EndpointMetaclass):
         if not cls.needs_body(None) or not hasattr(cls, 'model'):
             return {}
 
-        schema: typing.Dict[str, typing.Any] = {
-            'oneOf': []
-        }
+        schema: typing.Dict[str, typing.Any] = {}
+        oneOf: list[typing.Any] = []
         for model in typing.get_args(cls.model):
             if not inspect.isclass(model)\
             or not issubclass(model, pydantic.BaseModel):
@@ -430,7 +432,9 @@ class Endpoint(IEndpoint, metaclass=EndpointMetaclass):
                     f"{cls.__name__}.model must be a subclass of "
                     "pydantic.BaseModel or subclass thereof."
                 )
-            schema['oneOf'].append(model.schema())
+            oneOf.append(model.schema())
+        if oneOf:
+            schema['oneOf'] = oneOf
         if inspect.isclass(cls.model)\
         and issubclass(cls.model, pydantic.BaseModel):
             schema = cls.model.schema()
@@ -456,7 +460,10 @@ class Endpoint(IEndpoint, metaclass=EndpointMetaclass):
 
     @classmethod
     def needs_body(cls, method: typing.Optional[str]) -> bool:
-        return (method or cls.method) in cls.body_methods
+        methods: set[str] = set(getattr(cls, 'methods', None) or set())
+        if not methods and (method or cls.method):
+            methods = {method or cls.method} # type: ignore
+        return bool(methods & set(cls.body_methods))
 
     @classmethod
     def needs_parameter(cls, p: Parameter) -> bool:
