@@ -4,11 +4,13 @@ import copy
 import typing
 import fastapi
 import urllib.parse
+from typing import Any
 
 import pydantic
 import fastapi.responses
 from fastapi.exceptions import RequestValidationError
 from unimatrix.exceptions import CanonicalException
+from unimatrix.exceptions import CanonicalExceptionMetaclass
 
 from cbra.headers import CORS_HEADERS
 from cbra.headers import DIGEST_SCHEMA
@@ -20,11 +22,37 @@ from cbra.types import IRequestHandler
 from cbra.utils import DeferredException
 
 
-class BaseExceptionSpec(pydantic.BaseModel):
-    code: str
-    message: str
+class IExceptionSpec(pydantic.BaseModel):
+    code: str = pydantic.Field(
+        default=...,
+        title="Code",
+        description=(
+            "Further specifies the error condition in addition to the "
+            "`kind` parameter."
+        )
+    )
+
+    message: str = pydantic.Field(
+        default=...,
+        title="Message",
+        description=(
+            "A message describing the error. Note that this message "
+            "is not intended for display to the end-user."
+        )
+    )
+
+    hint: str | None = pydantic.Field(
+        default=None,
+        title="Hint",
+        description=(
+            "A hint describing possible actions to perform in order "
+            "to resolve the error condition."
+        )
+    )
+
+
+class BaseExceptionSpec(IExceptionSpec):
     detail: str
-    hint: str
 
     @classmethod
     def for_code(
@@ -117,14 +145,66 @@ class ExceptionModel(pydantic.BaseModel):
         example="v1"
     )
     metadata: ExceptionMetadata
+    spec: BaseExceptionSpec
+
+    @classmethod
+    def parse_exc(
+        cls,
+        exception_class: type['CanonicalException'],
+        data: dict[str, Any]
+    ) -> 'CanonicalException':
+        """Parse the exception data and return an instance of the given class."""
+        obj = cls.parse_obj(data)
+        return exception_class(id=obj.metadata.id, **obj.spec.dict())
 
 
-class CanonicalException(CanonicalException, ICanonicalException):
+class CanonicalExceptionMetaclass(CanonicalExceptionMetaclass):
+
+    def __new__(cls: type, name: str, bases: list[str], attrs: dict[str, Any]):
+        """Create a new :class:`CanonicalException` class."""
+        new = super().__new__
+        if attrs.pop('__abstract__', False):
+            if name == 'CanonicalException':
+                attrs['codes'] = {}
+            return new(cls, name, bases, attrs)
+
+        # Get the version and spec attributes to construct a pydantic model
+        # description the exception.
+        version: int = attrs.pop('version', 'v1')
+        spec: type[BaseExceptionSpec] = attrs.pop('spec', BaseExceptionSpec)
+        attrs['model'] = type(name, (ExceptionModel,), {
+            '__annotations__': {'spec': spec},
+            'spec': pydantic.Field(
+                default=...,
+                title=f'{name}Spec',
+                description="Error condition parameters."
+            )
+        })
+        return new(cls, name, bases, attrs)
+
+
+class CanonicalException(CanonicalException, metaclass=CanonicalExceptionMetaclass):
+    __abstract__: bool = True
+    codes: dict[str, Any]
+    model: type[ExceptionModel]
+    include_in_schema: bool = True
+    spec: type[IExceptionSpec] = BaseExceptionSpec
 
     @classmethod
     def defer(cls, **kwargs: typing.Any) -> DeferredException:
         return DeferredException(cls(**kwargs))
 
+    @staticmethod
+    def parse(code: str, data: dict[str, Any]) -> 'CanonicalException':
+        for cls in CanonicalException.registered:
+            if cls.code == code:
+                break
+        else:
+            cls = None
+        if cls is None or not hasattr(cls, 'spec'):
+            return None
+        return cls.model.parse_exc(cls, data)
+
     async def handle(
         self,
         request: fastapi.Request,
@@ -135,15 +215,56 @@ class CanonicalException(CanonicalException, ICanonicalException):
         instance.
         """
         if not renderer.has_content():
-            return fastapi.Response(status_code=self.http_status_code)
+            return fastapi.Response(
+                headers=self.get_http_headers(),
+                status_code=self.http_status_code
+            )
 
         return fastapi.Response(
             content=renderer.render(self.as_dict()),
+            headers=self.get_http_headers(),
             status_code=self.http_status_code,
             media_type=renderer.get_response_encoding()
         )
 
 
+class MissingScope(CanonicalException):
+    __module__: str = 'cbra.exceptions'
+    missing: set[str]
+    http_status_code: int = 401
+    code: str = "INVALID_SCOPE"
+    message: str = "The access credential did not have the required scope."
+    hint: str = (
+        "Request the scope described by the 'missing' parameter from the "
+        "authorization server."
+    )
+
+    class spec(IExceptionSpec):
+        missing: list[str] = pydantic.Field(
+            default=...,
+            title="Required scope",
+            description=(
+                "The list of scopes that the principal needs to obtain "
+                "to allow the request."
+            )
+        )
+
+    def __init__(self, missing: set[str], **kwargs: Any):
+        self.missing = missing
+        super().__init__(**kwargs)
+
+    def as_dict(self) -> dict[str, Any]:
+        dto = super().as_dict()
+        dto['spec']['missing'] = list(self.missing)
+        return dto
+
+    def upgrade_scope(self, scope: set[str]) -> set[str]:
+        """Return a set holding the input scope and the required
+        scope.
+        """
+        return set(self.missing) | scope
+
+
 class UncaughtException(CanonicalException):
     http_status_code: int = 500
 
@@ -300,7 +421,20 @@ class UpstreamServiceNotAvailable(CanonicalException):
         "Network or other infrastructure issues prevent "
         "proper operation of the service."
     )
-    hint = "Try again in 600 seconds."
+    hint = "Try again later."
+    retry_after: int | None = None
+
+    def __init__(self, retry_after: int | None = None, **kwargs: Any):
+        self.retry_after = retry_after
+        if retry_after is not None:
+            kwargs['hint'] = f'Try again in {retry_after} seconds.'
+        super().__init__(**kwargs)
+
+    def get_http_headers(self) -> dict[str, str]:
+        return  {
+            **super().get_http_headers(),
+            'Retry-After': str(self.retry_after or 120)
+        }
 
 
 class UpstreamConnectionFailure(UpstreamServiceNotAvailable):
