@@ -36,4 +36,7 @@ class AuthorizationCodeGrant(BaseGrant):
             "authorization request and their values must be identical, else "
             "this parameter must be absent from the request."
         )
-    )
\ No newline at end of file
+    )
+
+    def is_openid(self) -> bool:
+        return False
\ No newline at end of file
