@@ -12,7 +12,6 @@ import pydantic
 from fastapi.params import Query
 from pydantic.fields import FieldInfo
 
-from cbra.types import JSONQueryParameter
 from .iprincipal import IPrincipal
 from .redirecturl import RedirectURL
 from .responsetype import ResponseType
@@ -35,8 +34,9 @@ class BaseAuthorizationRequest:
     id_token_hint: str | None
     login_hint: str | None
     acr_values: str | None
+    amr_values: str | None
     registration: str | None
-    claims: JSONQueryParameter
+    claims: str | None
 
     @staticmethod
     def get_fields(
@@ -63,8 +63,9 @@ class BaseAuthorizationRequest:
             'id_token_hint': str | None,
             'login_hint': str | None,
             'acr_values': str | None,
+            'amr_values': str | None,
             'registration': str | None,
-            'claims': JSONQueryParameter,
+            'claims': str | None,
             'request_uri': str,
             'request': str
         }
@@ -300,6 +301,17 @@ class BaseAuthorizationRequest:
                     "The `acr` Claim is requested as a *Voluntary Claim* by this parameter."
                 )
             ),
+            'amr_values': field_factory(
+                default=None,
+                title="AMR values",
+                description=(
+                    "Requested **Authentication Method Reference** values. Space-separated "
+                    "string that specifies the `amr` values that the Authorization Server is "
+                    "being requested to use for processing this Authentication Request, with the "
+                    "values appearing in order of preference. The **Authentication Method Reference** "
+                    "satisfied by the authentication performed is returned as the `amr` Claim Value."
+                )
+            ),
             'registration': field_factory(
                 default=None,
                 title="Registration",
@@ -452,8 +464,11 @@ class BaseAuthorizationRequest:
     def get_acr_values(self) -> set[str]:
         return typing.cast(set[str], self.space_separated(self.acr_values))
 
+    def get_amr_values(self) -> set[str]:
+        return typing.cast(set[str], self.space_separated(self.amr_values))
+
     def get_claims(self) -> dict[str, typing.Any]:
-        return json.loads(self.claims)
+        return json.loads(self.claims or "null")
 
     def get_claims_locales(self) -> list[str]:
         return typing.cast(list[str], self.space_separated(self.claims_locales, True))
