@@ -2,6 +2,7 @@
 from typing import Any
 
 from ..exceptions import SubjectDoesNotExist # type: ignore
+from .iclient import IClient
 from .isubject import ISubject
 
 
@@ -15,8 +16,9 @@ class ISubjectRepository:
 
     async def get(
         self,
-        client: Any,
-        subject_id: Any
+        client: IClient,
+        subject_id: Any,
+        force_public: bool = False
     ) -> ISubject | None:
         """Return a :class:`ISubject` implementation for the client specified
         by `client_id` and `subject_id`. If `client_id` is ``None``, no properties
