@@ -7,6 +7,7 @@ from .authorizationcodegrant import AuthorizationCodeGrant
 from .basegrant import BaseGrant
 from .clientcredentialsgrant import ClientCredentialsGrant
 from .jwtbearerassertiongrant import JWTBearerAssertionGrant
+from .sessiongrant import SessionGrant
 
 
 class TokenRequestParameters(pydantic.BaseModel):
@@ -16,6 +17,7 @@ class TokenRequestParameters(pydantic.BaseModel):
         AuthorizationCodeGrant,
         ClientCredentialsGrant,
         JWTBearerAssertionGrant,
+        SessionGrant
     ]
 
     def get_root(self) -> BaseGrant:
